<?php

namespace app\api\controller;

use app\common\arw\adjfut\src\Validate;
use app\BaseController;
use app\common\logic\Login\CustomerLogin as LogicCustomerLogin;
use app\common\model\Token as ModelToken;
use app\Request;
use think\captcha\facade\Captcha;
use think\middleware\SessionInit;
use think\Response;
use think\facade\Db;

class Login extends BaseController
{
    /**
     * 初始化
     *
     * @date 2022-03-15
     * @example
     * @author admin
     * @since 1.0.0
     */
    protected function initialize(): void
    {
        $this->middleware[SessionInit::class] = [
            'only' => ['accountLogin', 'getCaptcha']
        ];
    }

    /**
     * 客户账号注册接口
     *
     * @param Request $request
     * @return array
     * @date 2023-06-25
     * @author xjh
     * @since 1.0.0
     */
    public function accountRegistration(Request $request): array
    {
        $params = Validate::param([
            'customer_name|客户昵称' => 'require',
            'customer_account|客户账号' =>  'require|alphaNum|min:10',
            'customer_password|客户密码' => 'require|alphaNum|min:8',
            // 'captcha|验证码' => $request->isProd() ? 'require|captcha'  : false
        ]);
        try {
            Db::startTrans();
            $token =  LogicCustomerLogin::accountRegistration($params);
            Db::commit();
            return [
                'code' => 0,
                'data' => [
                    'token' => $token->token_content,
                    'exp_time' => $token->token_exp_time,
                ],
                'msg' => '注册成功!'
            ];
        } catch (\Throwable $th) {
            Db::rollback();
            throwErrorMsg($th->getMessage());
        }
    }

    /**
     * 客户账号登录接口
     *
     * @param Request $request
     * @return array
     * @date 2023-06-25
     * @author xjh
     * @since 1.0.0
     */
    public function accountLogin(Request $request): array
    {
        $params = Validate::param([
            'customer_account|客户账号' =>  'require|alphaNum|min:10',
            'customer_password|客户密码' => 'require|alphaNum|min:8',
            // 'captcha|验证码' => $request->isProd() ? 'require|captcha'  : false
        ]);
        try {
            Db::startTrans();
            $token =  LogicCustomerLogin::accountLogin($params);
            Db::commit();
            return [
                'code' => 0,
                'data' => [
                    'token' => $token->token_content,
                    'exp_time' => $token->token_exp_time,
                ],
                'msg' => '登录成功!'
            ];
        } catch (\Throwable $th) {
            Db::rollback();
            throwErrorMsg($th->getMessage());
        }
    }

    /**
     * 客户登出接口
     *
     * @param Request $request
     * @return array
     * @date 2023-06-25
     * @author xjh
     * @since 1.0.0
     */
    public function logout(Request $request): array
    {
        ModelToken::logout();
        return msg('登出成功');
    }

    /**
     * 生成验证码
     *
     * @date 2023-06-25
     * @author xjh
     * @since 1.0.0
     */
    public function getCaptcha()
    {
        return Captcha::create();
    }
}
